"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Genotype = void 0;
class Genotype {
    constructor({ genetics, parentOnePair: parentOnePair, parentTwoPair: parentTwoPair, }) {
        if ([...parentOnePair, ...parentTwoPair].some((gene) => gene.feature !== parentOnePair[0].feature)) {
            throw new Error("genes within a genotype must have the same feature");
        }
        this._genetics = genetics;
        this._parentOnePair = parentOnePair;
        this._parentTwoPair = parentTwoPair;
        this._pair = [
            this._parentOnePair[this.random(this._parentOnePair.length)],
            this._parentTwoPair[this.random(this._parentTwoPair.length)],
        ];
    }
    get genetics() {
        return this._genetics;
    }
    get random() {
        return this._genetics.random;
    }
    get feature() {
        return this._parentOnePair[0].feature;
    }
    get parentOnePair() {
        return [...this._parentOnePair];
    }
    get parentTwoPair() {
        return [...this._parentTwoPair];
    }
    get pair() {
        return [...this._pair];
    }
    get phenotype() {
        const [geneOne, geneTwo] = this._pair;
        return geneOne.dominance > geneTwo.dominance ? geneOne : geneTwo;
    }
}
exports.Genotype = Genotype;
